@@ -22,7 +22,7 @@ class DrHistoryManager():
 		return self._is_saved
 
 	def empty_history(self):
-		"""Probably useless way to explicitely 'forget' the objects. It doesn't
+		"""Probably useless way to explicitly 'forget' the objects. It doesn't
 		really free the memory, but it kinda helps i suppose."""
 		for op in self._undo_history:
 			self._delete_operation(op)
@@ -60,7 +60,6 @@ class DrHistoryManager():
 	def can_undo(self):
 		# XXX incorrect si ya des states et qu'on redo
 		return (len(self._undo_history) > 0) or self._operation_is_ongoing()
-		# rarely called while an operation is ongoing so that ^ is stupid
 
 	def can_redo(self):
 		# XXX incorrect si ya des states ?
@@ -85,6 +84,13 @@ class DrHistoryManager():
 #			redo_label = redoable_action[0]['tool_id']
 #		self._image.window.update_history_actions_labels(undo_label, redo_label)
 
+	def rewind_history(self):
+		"""Put the entire 'undo' history into the 'redo' history, so the image
+		can be reset without losing any data."""
+		self._redo_history = self._undo_history[::-1] + self._redo_history
+		self._undo_history = []
+		self._image.update_history_sensitivity()
+
 	############################################################################
 	# Serialized operations ####################################################
 
@@ -109,7 +115,7 @@ class DrHistoryManager():
 		self.initial_operation = {
 			'tool_id': None,
 			'pixbuf': pixbuf,
-			'red': r, 'green': g, 'blue': b, 'alpha': a,
+			'rgba': Gdk.RGBA(red=r, green=g, blue=b, alpha=a),
 			'width': width, 'height': height
 		}
 
@@ -169,10 +175,11 @@ class DrHistoryManager():
 		# times less recomputation.
 
 	def _rebuild_from_history(self, async_cb_data={}):
-		"""Rebuild the image according to the content of the current history."""
+		"""Rebuild the image according to the content of the current history.
+		This is used as a GSourceFunc so it should return False."""
 		if not self._waiting_for_rebuild:
 			# It has already been rebuild by an other async call
-			return
+			return False
 		self._waiting_for_rebuild = False
 
 		last_save_index = self._get_last_state_index(True)
@@ -187,6 +194,7 @@ class DrHistoryManager():
 				# print("skip", op['tool_id'])
 				self._undo_history.append(op)
 		self._image.update()
+		return False
 
 	def _operation_is_ongoing(self):
 		return self._image.active_tool().has_ongoing_operation()
@@ -196,7 +204,7 @@ class DrHistoryManager():
 		if tool_id in all_tools:
 			return all_tools[tool_id]
 		else:
-			self._image.window.prompt_message(True, _("Error: no tool '%s'" % tool_id))
+			self._image.window.reveal_action_report(_("Error: no tool '%s'" % tool_id))
 			# no raise: this may happen normally if last_save_index is incorrect
 
 	############################################################################
