@@ -1,6 +1,6 @@
 # optionsbar_filters.py
 #
-# Copyright 2018-2021 Romain F. T.
+# Copyright 2018-2023 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 
 from gi.repository import Gtk
 from .abstract_optionsbar import AbstractOptionsBar
-from .utilities import utilities_add_unit_to_spinbtn
+from .utilities_units import utilities_add_unit_to_spinbtn
 
 class OptionsBarFilters(AbstractOptionsBar):
 	__gtype_name__ = 'OptionsBarFilters'
@@ -32,7 +32,7 @@ class OptionsBarFilters(AbstractOptionsBar):
 
 		self.menu_btn = builder.get_object('menu_btn')
 		self._togglable_btn = self.menu_btn
-		self.menu_btn.set_menu_model(self.filters_tool.get_filters_menu())
+		self.menu_btn.set_menu_model(self.filters_tool.get_options_model())
 		self._menu_label = builder.get_object('menu_label')
 		self._menu_icon = builder.get_object('menu_icon')
 
@@ -64,7 +64,7 @@ class OptionsBarFilters(AbstractOptionsBar):
 
 	def on_filter_changed(self):
 		self.set_compact(self._is_narrow)
-		self.window.set_picture_title()
+		self.window.set_window_subtitles()
 		# self.menu_label.set_label(self.filters_tool.type_label) # XXX width???
 
 	def set_compact(self, state):
