@@ -1,6 +1,6 @@
 # abstract_tool.py
 #
-# Copyright 2018-2021 Romain F. T.
+# Copyright 2018-2023 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import cairo
-from gi.repository import Gtk
+from gi.repository import Gtk, Gdk
 
 class WrongToolIdException(Exception):
 	def __init__(self, expected, actual):
@@ -37,8 +37,9 @@ class AbstractAbstractTool():
 		# The tool's identity
 		self.id = tool_id
 		self.menu_id = 0
-		self.label = label
-		self.icon_name = icon_name
+		self.label = self._mnemo_label = label
+		self._mnemo_char = ""
+		self._icon_name = icon_name
 		# The options it supports
 		self.accept_selection = False
 		self.use_color = False
@@ -46,8 +47,8 @@ class AbstractAbstractTool():
 		# The tool's state
 		self.cursor_name = 'cell'
 		self._ongoing_operation = False
+		self._modifier_keys = []
 		# Once everything is set, build the UI
-		self.build_row()
 		self.try_build_pane()
 
 	############################################################################
@@ -86,14 +87,29 @@ class AbstractAbstractTool():
 		self.set_action_sensitivity('exchange_color', self.use_color)
 		self.set_action_sensitivity('cairo_operator', self.use_operator)
 
+		self.set_action_sensitivity('cancel_transform', False)
+		self.set_action_sensitivity('apply_transform', False)
+
 	def get_settings(self):
 		return self.window.options_manager._tools_gsettings
 
+	def update_modifier_state(self, event_state):
+		modifier_keys = []
+		# CONTROL_MASK can't be used, because it already has an effect app-wide.
+		if (event_state & Gdk.ModifierType.SHIFT_MASK) == Gdk.ModifierType.SHIFT_MASK:
+			modifier_keys.append('SHIFT')
+		if (event_state & Gdk.ModifierType.MOD1_MASK) == Gdk.ModifierType.MOD1_MASK:
+			modifier_keys.append('ALT')
+		self._modifier_keys = modifier_keys
+
 	############################################################################
 	# Various utilities ########################################################
 
 	def show_error(self, error_text):
-		self.window.prompt_message(True, error_text)
+		self.window.reveal_message(error_text)
+
+	def get_tooltip(self, event_x, event_y, motion_behavior):
+		return None
 
 	############################################################################
 	# Bottom pane and menubar integration ######################################
@@ -104,7 +120,10 @@ class AbstractAbstractTool():
 	def build_bottom_pane(self):
 		return None
 
-	def on_apply_temp_pixbuf_tool_operation(self, *args):
+	def on_apply_transform_tool_operation(self, *args):
+		pass # implemented only by transform tools
+
+	def on_cancel_transform_tool_operation(self, *args):
 		pass # implemented only by transform tools
 
 	def get_options_label(self):
@@ -114,7 +133,7 @@ class AbstractAbstractTool():
 		pass
 
 	def add_item_to_menu(self, tools_menu):
-		tools_menu.append(self.label, 'win.active_tool::' + self.id)
+		tools_menu.append(self._mnemo_label, 'win.active_tool::' + self.id)
 
 	def get_options_model(self):
 		"""Returns a Gio.MenuModel corresponding to the tool's options. It'll be
@@ -131,45 +150,84 @@ class AbstractAbstractTool():
 		menu can't provide all the features."""
 		return None
 
-	def get_edition_status(self):
-		return self.label
+	def get_editing_tips(self):
+		return [self.label]
 
 	############################################################################
 	# Side pane ################################################################
 
-	def build_row(self):
-		"""Build the GtkRadioButton for the sidebar. This method stores it as
-		'self.row', but does not pack it in the bar, and does not return it."""
-		self.row = Gtk.RadioButton(relief=Gtk.ReliefStyle.NONE, \
-		                        draw_indicator=False, valign=Gtk.Align.CENTER, \
-		                                                tooltip_text=self.label)
-		self.row.set_detailed_action_name('win.active_tool::' + self.id)
-		self.label_widget = Gtk.Label(label=self.label) #, use_underline=True)
+	def set_mnemonics(self, character):
+		self._mnemo_label = self.label.replace(character, "_" + character, 1)
+		self._mnemo_char = character.upper()
+
+	def build_flowbox_child(self, flowbox):
+		"""Build the icon and its label for the sidebar."""
+		# The icon
 		if self.window.gsettings.get_boolean('big-icons'):
 			size = Gtk.IconSize.LARGE_TOOLBAR
 		else:
 			size = Gtk.IconSize.SMALL_TOOLBAR
-		image = Gtk.Image().new_from_icon_name(self.icon_name, size)
-		box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=8)
-		box.add(image)
-		box.add(self.label_widget)
-		self.row.add(box)
-		self.row.show_all()
+		image = Gtk.Image().new_from_icon_name(self._icon_name, size)
+		self._label_box = Gtk.Box( \
+			orientation=Gtk.Orientation.HORIZONTAL, \
+			spacing=8, \
+			margin=8 \
+		)
+		self._label_box.add(image)
+
+		# The readable label
+		label_widget = Gtk.Label(use_underline=True, label=self._mnemo_label)
+		self._label_box.add(label_widget)
+		self._label_box.show_all()
+
+		# The "mini-label" is shown only when pressing <Alt>
+		mini_label = Gtk.Label(use_underline=True, label="_" + self._mnemo_char)
+		self._label_box.add(mini_label)
+
+		flowbox.add(self._label_box)
+		self._fb_child = self._label_box.get_parent()
+
+		# The item's tooltip
+		if self._mnemo_char == "":
+			self._fb_child.set_tooltip_text(self.label)
+		else:
+			tooltip = self.label + " (Alt+" + self._mnemo_char + ")"
+			self._fb_child.set_tooltip_text(tooltip)
+
+	def select_flowbox_child(self, *args):
+		self.window.tools_flowbox.select_child(self._fb_child)
+
+	def is_flowbox_child_selected(self):
+		return self._fb_child.is_selected()
 
 	def set_show_label(self, label_visible):
-		self.label_widget.set_visible(label_visible)
+		label_widget = self._label_box.get_children()[1]
+		label_widget.set_visible(label_visible)
+		icon = self._label_box.get_children()[0]
 		if label_visible:
-			self.row.get_children()[0].set_halign(Gtk.Align.START)
+			icon.set_halign(Gtk.Align.START)
 		else:
-			self.row.get_children()[0].set_halign(Gtk.Align.CENTER)
+			icon.set_halign(Gtk.Align.CENTER)
 
 	def update_icon_size(self):
-		image = self.row.get_children()[0].get_children()[0]
+		image = self._label_box.get_children()[0]
 		if self.window.gsettings.get_boolean('big-icons'):
 			size = Gtk.IconSize.LARGE_TOOLBAR
 		else:
 			size = Gtk.IconSize.SMALL_TOOLBAR
-		image.set_from_icon_name(self.icon_name, size)
+		image.set_from_icon_name(self._icon_name, size)
+
+	def show_only_mnemonics(self, should_show):
+		if self._mnemo_char == "":
+			return
+		label_widget = self._label_box.get_children()[1]
+		if label_widget.get_visible():
+			return
+		image = self._label_box.get_children()[0]
+		mini_label = self._label_box.get_children()[2]
+
+		image.set_visible(not should_show)
+		mini_label.set_visible(should_show)
 
 	############################################################################
 	# Activation or not ########################################################
@@ -181,6 +239,8 @@ class AbstractAbstractTool():
 		pass
 
 	def cancel_ongoing_operation(self):
+		"""Reset the current tool when 'undo' is pressed while an operation has
+		been started but not applied."""
 		self.on_tool_unselected()
 		self.give_back_control(self.accept_selection) # XXX pas sûr
 		self.on_tool_selected()
@@ -189,6 +249,9 @@ class AbstractAbstractTool():
 		self._ongoing_operation = False
 
 	def give_back_control(self, preserve_selection):
+		"""Reset the tool's state well enough to allow for a new tool to start.
+		It should include unselecting/applying the selection if the next tool
+		doesn't support it."""
 		self.restore_pixbuf()
 		self.non_destructive_show_modif()
 
@@ -269,7 +332,7 @@ class AbstractAbstractTool():
 	def on_press_on_area(self, event, surface, event_x, event_y):
 		pass
 
-	def on_motion_on_area(self, event, surface, event_x, event_y):
+	def on_motion_on_area(self, event, surface, event_x, event_y, render=True):
 		pass
 
 	def on_unclicked_motion_on_area(self, event, surface):
