@@ -1,6 +1,6 @@
 # tool_rotate.py
 #
-# Copyright 2018-2021 Romain F. T.
+# Copyright 2018-2023 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -45,11 +45,23 @@ class ToolRotate(AbstractCanvasTool):
 		self.angle_btn.connect('value-changed', self.on_angle_changed)
 		return pane
 
-	def get_edition_status(self):
+	def get_options_label(self):
+		return _("Rotating options")
+
+	def get_editing_tips(self):
 		if self.apply_to_selection:
-			return _("Rotating the selection")
+			label_action = _("Rotating the selection")
+			label_confirm = None
 		else:
-			return _("Rotating the canvas")
+			label_action = _("Rotating the canvas")
+			label_confirm = self.label + " - " + \
+			                         _("Don't forget to confirm the operation!")
+
+		# there is intentionally no `label_modifier_shift` because it would be
+		# too hard to explain the actions of each mouse button.
+
+		full_list = [label_action, label_confirm]
+		return list(filter(None, full_list))
 
 	def on_tool_selected(self, *args):
 		super().on_tool_selected()
@@ -59,6 +71,7 @@ class ToolRotate(AbstractCanvasTool):
 		self.build_and_do_op() # Show the temp_pixbuf before any event
 		if self.apply_to_selection:
 			self.cursor_name = 'move'
+			# not the ideal cursor, but their is no ideal cursor for this
 		else:
 			self.cursor_name = 'pointer'
 		# the pane is updated by the window according to self.apply_to_selection
@@ -66,19 +79,27 @@ class ToolRotate(AbstractCanvasTool):
 	############################################################################
 
 	def on_press_on_area(self, event, surface, event_x, event_y):
-		if not self.apply_to_selection:
+		self.update_modifier_state(event.state)
+		if 'SHIFT' in self._modifier_keys:
+			if event.button == 1:
+				self.on_horizontal_clicked()
+			elif event.button == 3:
+				self.on_vertical_clicked()
+			return
+		elif not self.apply_to_selection:
 			if event.button == 1:
 				self.on_left_clicked()
 			elif event.button == 3:
 				self.on_right_clicked()
 			return
+
 		center_x, center_y = self.get_selection().get_center_coords()
 		delta_x0 = center_x - event_x
 		delta_y0 = center_y - event_y
 		press_as_degrees = (math.atan2(delta_x0, delta_y0) * 180) / math.pi
 		self.angle_press = self.get_angle() - int(press_as_degrees)
 
-	def on_motion_on_area(self, event, surface, event_x, event_y):
+	def on_motion_on_area(self, event, surface, event_x, event_y, render=True):
 		if not self.apply_to_selection:
 			return
 		center_x, center_y = self.get_selection().get_center_coords()
@@ -86,8 +107,9 @@ class ToolRotate(AbstractCanvasTool):
 		delta_y = center_y - event_y
 		release_angle = ( math.atan2(delta_x, delta_y) * 180 ) / math.pi
 		self.angle_btn.set_value(int(release_angle) + self.angle_press)
-		operation = self.build_operation()
-		self.do_tool_operation(operation)
+		if render:
+			operation = self.build_operation()
+			self.do_tool_operation(operation)
 
 	def on_release_on_area(self, event, surface, event_x, event_y):
 		self.on_motion_on_area(event, surface, event_x, event_y)
@@ -166,9 +188,7 @@ class ToolRotate(AbstractCanvasTool):
 		# print('angle:', angle)
 		# print('gdk_rotation:', gdk_rotation)
 		# print('cairo_rotation:', cairo_rotation)
-
-		# Image rotation, using the method from GdkPixbuf.Pixbuf
-		new_pixbuf = source_pixbuf.rotate_simple(gdk_rotation)
+		new_pixbuf = source_pixbuf
 
 		# Image flipping (horizontal or vertical "mirroring")
 		if flip_h:
@@ -176,6 +196,9 @@ class ToolRotate(AbstractCanvasTool):
 		if flip_v:
 			new_pixbuf = new_pixbuf.flip(False)
 
+		# Image rotation, using the method from GdkPixbuf.Pixbuf
+		new_pixbuf = new_pixbuf.rotate_simple(gdk_rotation)
+
 		# Image rotation, using methods from cairo.Context (only if needed)
 		if cairo_rotation != 0:
 			surface0 = Gdk.cairo_surface_create_from_pixbuf(new_pixbuf, 0, None)
